# shellcheck shell=sh
# shellcheck disable=SC2034 
# shellcheck disable=SC2059
# shellcheck disable=SC1091
# shellcheck disable=SC1090

trap fo_close INT QUIT TERM EXIT

FO_SCRIPT="$(basename "${0}")"
readonly FO_SCRIPT
readonly RED="\033[1;41m"
readonly NORM="\033[0m"
readonly UNMOUNTED="# "

auxilium_add nolock  -t opt -m "Don\'t create lock file"

another_instance() {
  logger -t "udev" "===> There is another instance running, exiting"
  exit 1
}

fo_repeat_char(){
  # repeat a char X 
	# param: char to repeat
	# param: nb of repeat
	c="$1"
	# shellcheck disable=SC2086
	printf -- "${c}%.0s" $(seq 1 ${2})
}

fo_contain(){
	# check if a string ($1) contain another ($2)
	# param: String to check if contain another
	# param: String to check if is in another
	[ "$#" -lt 2 ] && return 1
	case " $1 " in
		*" ${2} "*)
			echo 1
			;;
		*)
			echo 0
			;;
	esac
}

fo_center_align(){
	# Center align a text
	# Param: Text to align
	# Param: len of final string
	# Param: If is disk to locate
	
	lenTotal="$1"
	locate="$2"
	text="$3"
	[ -z "$text" ] && text='Unknown'
	lenText=${#text}
	lenTotal=$((lenTotal + 2))
	nb_spaces=$((lenTotal - lenText))
	space=' '
	color_begin=''
	color_end=''
	if [ "$locate" = 1 ];then
		if [ -z "$AUXILIUM_ARG_redmine" ]; then
			color_begin=${RED}
			color_end=${NORM}
		else
			space='='
		fi
	fi
	first_nb=$((nb_spaces/2))
	first_spaces=$(fo_repeat_char "${space}" ${first_nb})
	end_nb=$((nb_spaces - first_nb))
	end_spaces=$(fo_repeat_char "${space}" ${end_nb})
	ret=$(printf -- "%s%s%s" "${first_spaces}" "$text" "${end_spaces}")
	printf -- "${color_begin}%${lenTotal}s${color_end}" "$ret"
}

fo_get_dev_letter(){
	# Print disk letter (a to z) by is index 
	# Param: Disk Index (beginning at 0)
	num=$((97 + ${1}))
	printf "\\$(printf '%03o' ${num})"
}

fo_get_dev_ascii(){
	# Print disk ascii code of last letter of device name
	# Param: letter
	name_id="$(printf "%d\n" "$(printf '%s' "$1" | cut -c 1)")"
	ret=$((name_id - 97))
	echo $ret
}

fo_lock() {
	LOCKDIR=/var/run/lock
	FO_LOCKFILE=${LOCKDIR}/$(basename "${0}")
	readonly FO_LOCKFILE
	if [ -z "$AUXILIUM_ARG_nolock" ] ; then
		logger -t "udev" "===> Claming lock for $0 in ${FO_LOCKFILE}"
		if [ -f "$FO_LOCKFILE" ]; then
			kill -0 "$(cat "$FO_LOCKFILE")" && another_instance
		fi
		echo $$ > "$FO_LOCKFILE"
	fi
}

fo_use_cache_file() {
	# Test if a cache file is valid, and delete it if invalid
	# Param: Cache File Suffix
	usecache=false
	# read a cache file, if exist and recent
	if [ -f "$1" ]; then
		lastentry=$(stat -c %Y "$1")
		lastvalid=$(date -d "now -5 min" +%s)
		if [ "$lastentry" -gt "$lastvalid" ]; then
			usecache=true
		else
			rm "$1"
		fi
	elif [ -e "$1" ]; then
		rm "$1"
	fi
	if ! $usecache; then
		logger -t "udev" "[Rebuilding cache file]"
		# We should rebuild the cache: exit 1
		return 1
	else
		logger -t "udev" "[Using cache file]"
		# We can re-use the cache: exit 0
		return 0
	fi
}

fo_check_valid(){
	v=$(dpkg-query -W lshw|cut -f2)
	dpkg --compare-versions "${v}" ge 02.19 || readonly FO_LSHW_VALID=$?
  v=$(dpkg-query -W smartmontools|cut -f2)
	dpkg --compare-versions "${v}" ge 7.2 || readonly FO_SMARTCTL_VALID=$?
  v=$(dpkg-query -W jq|cut -f2)
	dpkg --compare-versions "${v}" ge 1.5 || readonly FO_JQ_VALID=$?
	v=$(dpkg-query -W storcli|cut -f2)
	dpkg --compare-versions "${v}" ge 7.2508 || readonly FO_STORCLI_VALID=$?
}

fo_load_storcli(){
	if [ -z "$FO_JQ_VALID" ] && [ -z "$FO_STORCLI_VALID" ]; then
		fo_use_cache_file "${FO_CACHE_FILE_STORCLI}" || ${STORCLI} /c0 show nolog J|jq -r '.["Controllers"][]["Response Data"]' > "${FO_CACHE_FILE_STORCLI}"
	fi
}

fo_check_raid(){
	[ "$1" = "-f" ] && return 0
  [ "$(echo "$ID_MODEL"|cut -d_ -f1)" = "RAID" ] && return 0
  [ "$(echo "$ID_MODEL"|cut -d_ -f1)" = "PERC" ] && return 0
	return 1
}

fo_close() {
  auxilium_close
	rm "$FO_LOCKFILE" 2> /dev/null || true
}

fo_init(){
	if [ -f "/etc/ferreus-orbit/ferreus-orbit.conf" ] ; then
		. "/etc/ferreus-orbit/ferreus-orbit.conf"
	elif [ -f "etc/ferreus-orbit.conf" ] ; then
		. "etc/ferreus-orbit.conf"
	else
		echo "Error: could not load ferreus-orbit.conf"
		exit 1
	fi

	FO_DEFAULT_RAID0=0

	readonly LSHW=/usr/bin/lshw
	readonly FO_CACHE_FILE_LSHW=/tmp/fo_cache_lshw
	readonly FO_CACHE_FILE_STORCLI=/tmp/fo_cache_storcli
	readonly FO_CACHE_DISKS_INFOS=/tmp/fo_disks_infos

	readonly VAR_PATH='/var/cache/ferreus-orbit/'

	if [ -x /usr/bin/storcli64 ] ; then
		readonly STORCLI=/usr/bin/storcli64
	elif [ -x /usr/bin/storcli_x64 ] ; then
		readonly STORCLI=/usr/bin/storcli_x64
	elif [ -x /usr/bin/storcli ] ; then
		readonly STORCLI=/usr/bin/storcli
	fi

	readonly FO_ETC_PLUGINS_PATH="/etc/ferreus-orbit/plugins"
	if [ -d "/usr/share/ferreus-orbit/plugins" ]; then
		readonly FO_PLUGINS_PATH="/usr/share/ferreus-orbit/plugins"
	elif [ -d "share/plugins" ] ; then
		readonly FO_PLUGINS_PATH="share/plugins"
	else
		readonly FO_PLUGINS_PATH="../share/plugins"
	fi

	FO_CHASSIS_MANUFACTURER=$(dmidecode -s system-manufacturer)
	readonly FO_CHASSIS_MANUFACTURER
	FO_CHASSIS_MODEL=$(dmidecode -s system-product-name)
	readonly FO_CHASSIS_MODEL

	fo_check_valid
	if [ -d /dev/disk/oci-sort ] ; then
		USE_FO_NAME=0
		readonly SORTED_PATH='/dev/disk/oci-sort'
	elif [ -d /dev/disk/fo-sort ] ; then
		USE_FO_NAME=1
		readonly SORTED_PATH='/dev/disk/fo-sort'
	else
		USE_FO_NAME=-1
		readonly SORTED_PATH='/dev'
	fi
	if [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
	elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
	elif [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
	elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
	else
		. "${FO_PLUGINS_PATH}/generic"
	fi
}
